
bool reset;
float3 gravity;
int pCount;

//Ground:
float groundYLevel = 0.0;
float groundBounce = 1;

float velocityDamping = 0.95;
float velocityDampingRadius = 0.05;

//RandomDirectionBuffer
StructuredBuffer<float3> randomVector;	// random -1..1
int brwIndexShift;
float brwnStrength;
float timeDifference;

struct particle
{
	float3 pos;
	float3 vel;
	float4 col;
	float scale;
	float2 lifeFromTo;
	int index;
//	float3 rotation;
};
RWStructuredBuffer<particle> Output : BACKBUFFER;

//==============================================================================
//COMPUTE SHADER ===============================================================
//==============================================================================

[numthreads(64, 1, 1)]
void CSConstantForce( uint3 DTid : SV_DispatchThreadID )
{
	if (reset)
	{
		Output[DTid.x].pos = randomVector[DTid.x].xyz*10;
		Output[DTid.x].vel = 0;
	}

	else
	{
		float3 p = Output[DTid.x].pos;
		float3 v = Output[DTid.x].vel;

		//Velocity Damping:
		v *= randomVector[DTid.x].x * velocityDampingRadius + velocityDamping;

		// Brownian
		if (brwnStrength!=0.0) {
			uint rndIndex = (DTid.x + brwIndexShift) % pCount;
			float3 brwnForce = randomVector[rndIndex];
			v += brwnForce * brwnStrength;	
		}
		
		//Ground:
		if(p.y < groundYLevel) 
		{
			v = groundBounce * reflect(v, float3(0,1,0));
			p.y = groundYLevel;
		}
		//Gravity: 
		v += gravity;

		Output[DTid.x].vel = v;
		Output[DTid.x].pos = p + v * timeDifference;
	}
}

//==============================================================================
//TECHNIQUES ===================================================================
//==============================================================================

technique11 simulation
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CSConstantForce() ) );
	}
}
