
int pCount;
int forceCount = 0;

//Forces
StructuredBuffer<float3> forceMultiplyXYZ;
//Transform Space to Texture
StructuredBuffer<float4x4> XYZtoUVtransform;
//Go Back;
StructuredBuffer<float4x4> UVtoXYZtransform;

//UV Correction;
float4x4 UVcorrection;

float colorAdapt = .0;
float positionAdapt = 1.0;
float filterKinectBlack = 0.0;


//Texture source
Texture2D tex <string uiname="Height Texture";>;
Texture2D texRGB <string uiname="Color Texture";>;

SamplerState mySampler : IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

struct particle
{
	float3 pos;
	float3 vel;
	float4 col;
	float scale;
	float2 lifeFromTo;
	int index;
//	float3 rotation;
};
RWStructuredBuffer<particle> Output : BACKBUFFER;

//==============================================================================
//COMPUTE SHADER ===============================================================
//==============================================================================

[numthreads(64, 1, 1)]
void CSConstantForce( uint3 DTid : SV_DispatchThreadID )
{
	
	float3 p = Output[DTid.x].pos;
	float3 v = Output[DTid.x].vel;
	float3 newP = p;
	float3 samplePos;
	
	for (int i=0; i<forceCount; i++) {
	
		samplePos = mul(float4(p,1), XYZtoUVtransform[0]).xyz;
		
		// Sample all colors to height
		samplePos.z = length( tex.SampleLevel(mySampler,samplePos.xy,0).rgb );
		
		// Compute position back
		newP = mul(float4(samplePos,1), UVtoXYZtransform[0]).xyz;
		
		// correct (RGB Camera - IR Camera) distance in texture space
		samplePos = mul(float4(samplePos,1), UVcorrection).xyz;
		
		// Get Color
		float4 textureColor = texRGB.SampleLevel(mySampler,samplePos.xy,0);
		Output[DTid.x].col = lerp(Output[DTid.x].col, textureColor, colorAdapt);
	}
	

	//Output[DTid.x].vel = v;
	
	// Only i there is a position inormation, adapt the position.
	if (samplePos.z>filterKinectBlack)
	Output[DTid.x].pos = lerp( p, newP, positionAdapt);
	
}

//==============================================================================
//TECHNIQUES ===================================================================
//==============================================================================

technique11 heightPosition
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CSConstantForce() ) );
	}
}
