
int pCount;
int attrCount = 0;

//Attractors Position Buffer
StructuredBuffer<float3> attrPos;
//Attractors Radius
StructuredBuffer<float> attrRadius;
//Attractors Force
StructuredBuffer<float> attrForce;
//Attractors Up Force
StructuredBuffer<float> attrUpForce;

float torque = 0.5;

struct particle
{
	float3 pos;
	float3 vel;
	float4 col;
	float scale;
	float2 lifeFromTo;
	int index;
//	float3 rotation;
};
RWStructuredBuffer<particle> Output : BACKBUFFER;

//==============================================================================
//COMPUTE SHADER ===============================================================
//==============================================================================

[numthreads(64, 1, 1)]
void CSConstantForce( uint3 DTid : SV_DispatchThreadID )
{
	{
		float3 p = Output[DTid.x].pos;
		float3 v = Output[DTid.x].vel;
		
		float dist;
		float force;
		for(int i=0 ; i<attrCount; i++) {
			dist = distance(p, attrPos[i]) / attrRadius[i];

			if ( dist < 1 && dist > 0 ) {
				// Push UP
				float3 upVec;
				if (i<attrCount-1) upVec = normalize ( attrPos[i+1]-attrPos[i] );
				else upVec = float3(0,1,0);
				
				// Attrator
				float3 toAttrVec = normalize(p - attrPos[i]);
				force = smoothstep( 0, 1, 1 - abs(2*dist-1) );
				v += attrForce[i] * force * toAttrVec;
			
				// Force UP
				v += attrUpForce[i] * force * upVec;
				
				// Torque
				v += torque * (1/(dist+0.2)) * force * cross(upVec,toAttrVec);
			}
			
			
		}
/*	
		//Multiple attractors
		uint count,dummy;	
		attrPos.GetDimensions(count,dummy);
			//attrVec = p - attrBuffer[i];
			float3 attrVec = attrPos[i] - p;
			float attrRadius = attrData[i].x;
			float attrStrength = attrData[i].y;

			float attrForce = length(attrVec) / attrRadius;
			attrForce = 1 - attrForce;
			attrForce = saturate(attrForce);
			attrForce = pow(attrForce, 2);
			attrVec = attrVec * attrForce * attrStrength;
			//transform attraction vector:
			attrVec = mul(float4(attrVec,1), attrForceT).xyz;
			v += attrVec;
		}
*/
		
		Output[DTid.x].vel = v;
	}
}

//==============================================================================
//TECHNIQUES ===================================================================
//==============================================================================

technique11 whirlwind
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CSConstantForce() ) );
	}
}
