int pCount;
int emitCount;
float4x4 tW: WORLD;        //the models world matrix


//Emitters Position Buffer
float2 positionRadius = 0;
float randomDirection = .2;
float randomVelocity = .2;
float velocityMultiple = 1;

//Emitters Color
StructuredBuffer<float4> emitCol;

StructuredBuffer<float4> emitRndDir;

StructuredBuffer<float3> geometryPosition;
StructuredBuffer<float3> geometryNormal;


int indexOffset;
//int ranomOffset;
float currentTime = 0;
float lifeTime = 1; // in  seconds
float lifeTimeRandom = 0.5; // is applied to a squared random 0..1

struct particle
{
	float3 pos;
	float3 vel;
	float4 col;
	float scale;
	float2 lifeFromTo;
	int index;
//	float3 rotation;
};

RWStructuredBuffer<particle> Output : BACKBUFFER;

/*struct geom
{
	float3 pos;
	float3 norm;
	float2 UVpos;
};*/

//==============================================================================
//COMPUTE SHADER ===============================================================
//==============================================================================

[numthreads(1, 1, 1)]
void CSCloudEmiter( uint3 DTid : SV_DispatchThreadID )
{
	
	// Emitters Data:
	uint emitIndex = DTid.x % emitCount;
	// indices of emitted particles:
	uint index = ( indexOffset + DTid.x ) % pCount;
	//uint randomIndex = ( ranomOffset + DTid.x ) % pCount;
	
	
	uint count,dummy;	
	geometryPosition.GetDimensions(count,dummy);
	float3 p = geometryPosition[floor(emitRndDir[index].w*count)];
	float3 v = geometryNormal[floor(emitRndDir[index].w*count)];

	// mathematical function to create uniform random distribution on a sphere
	float theta0 = 2 * 3.14159265 * emitRndDir[index].x;
	float theta1 = acos( 1 - 2*emitRndDir[index].y );
	float3 SpherePos;
	SpherePos.x = cos(theta0)*sin(theta1);
	SpherePos.y = sin(theta0)*sin(theta1);
	SpherePos.z = cos(theta1);
	
	// get position radius 	// F + (T-F)*(0..1)
	float radiusMultiple = positionRadius.x + (positionRadius.y-positionRadius.x) * (1-emitRndDir[index].z);
	
	// random
	v = (v + SpherePos*randomDirection) * (1-randomVelocity*emitRndDir[index].z);
	
	float4 c = emitCol[emitIndex];

	// write position and velocity of emitted particles:
	Output[index].vel = v * velocityMultiple;
	Output[index].pos = mul(float4( p + SpherePos * radiusMultiple ,1),tW).xyz;
	Output[index].col = c;
	Output[index].scale = 1.0;
	Output[index].lifeFromTo.xy = float2( currentTime, currentTime + lifeTime + emitRndDir[index].z*emitRndDir[index].z*lifeTimeRandom );
	Output[index].index = index;
}

//==============================================================================
//TECHNIQUES ===================================================================
//==============================================================================

technique11 cloudEmission
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CSCloudEmiter() ) );
	}
}
