int pCount;
int emitCount;


//Emitters Color
StructuredBuffer<float4> emitCol;

StructuredBuffer<float3> emitRndDir;

int indexOffset;
float currentTime = 0;
float lifeTime = 1.0; // in seconds
float4x4 Transformation;

float3 quadFrom = {-1.0, -1.0, .0};
float3 quadScale = {2.0, 2.0, .0};



//Texture we want to read from
Texture2D tex <string uiname="Texture";>;

//Buffer containing uvs for sampling
//StructuredBuffer<float2> uv <string uiname="UV Buffer";>;

SamplerState mySampler : IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};
//rwbuffer[i.x] = tex.SampleLevel(mySampler,uv[i.x],0);


struct particle
{
	float3 pos;
	float3 vel;
	float4 col;
	float scale;
	float2 lifeFromTo;
	int index;
//	float3 rotation;
};

RWStructuredBuffer<particle> Output : BACKBUFFER;

//==============================================================================
//COMPUTE SHADER ===============================================================
//==============================================================================

[numthreads(1, 1, 1)]
void CSCloudEmiter( uint3 DTid : SV_DispatchThreadID )
{
	
	// Emitters Data:
	uint emitIndex = DTid.x % emitCount;
	// indices of emitted particles:
	uint index = indexOffset + DTid.x;
	index = index % pCount;
	
		// (3d) vector of new (3d)random position 	// (B-A)*(0..1)
	float3 p = quadFrom + quadScale * emitRndDir[index];
	p = mul(float4(p,1.0), Transformation).xyz;
	
	float4 c = emitCol[emitIndex];
	
	// both position and color is derived from emitRndDir
	float4 textureColor = tex.SampleLevel(mySampler,float2(emitRndDir[index].x,1-emitRndDir[index].y),0);

	// write position and velocity of emitted particles:
	Output[index].vel = 0;
	Output[index].pos = p;
	Output[index].col = c*textureColor;
	Output[index].scale = 1.0;
	Output[index].lifeFromTo.xy = float2( currentTime, currentTime + lifeTime);
	Output[index].index = index;
}

//==============================================================================
//TECHNIQUES ===================================================================
//==============================================================================

technique11 cloudEmission
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CSCloudEmiter() ) );
	}
}
